#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

/* Funkcija ispisuje poruku i potom informaciju o greski na osnovu errno vrednosti. */
void error_fatal (const char *message){
  perror (message);
  exit (1);
}

/* Program stampa tip fajla za svaki fajl cije je ime navedeno u komandnoj liniji.*/
int main(int argc, char **argv){
  struct stat buffer;
  int i;

  /* Proverava se da li postoji bar 1 argument komandne linije. */
  if (argc < 2){
    fprintf (stderr, "At least one file path needed as the argument of the program.\n");
    exit (EXIT_FAILURE);
  }

  /* Prolazi se kroz argumente komandne linije. */
  for (i=1; i<argc; i++){

    /* Koristi se funkcija lstat a ne stat jer daje informacije o simbolickom linku
       a ne o fajlu na koji pokazuje taj link. */ 
    if (lstat (argv[i], &buffer) < 0){
      /* Kreira se poseban string kako bi se lakse uocilo kod kog fajla je doslo do greske. */
      char error_message [256];
      sprintf (error_message, "lstat error for %s", argv[i]);
      error_fatal (error_message);
    }

    printf ("%s : ", argv[i]);

    /* Proverava se tip fajla pomocu definisanih makroa. */
    if (S_ISREG (buffer.st_mode))
      printf ("regular\n");
    else if (S_ISDIR (buffer.st_mode))
      printf ("directory\n");
    else if (S_ISCHR (buffer.st_mode))
      printf ("character device\n");
    else if (S_ISBLK (buffer.st_mode))
      printf ("block device\n");
    else if (S_ISFIFO (buffer.st_mode))
      printf ("fifo\n");
    else if (S_ISLNK (buffer.st_mode))
      printf ("symbolic link\n");
    else if (S_ISSOCK (buffer.st_mode))
      printf ("socket\n");
    else
      error_fatal ("Unknown type of file");
  }

  return 0;
}
